#!/usr/bin/perl

# Don't forget to add version and intellectual property control information.

# The %type_info_all hash was automatically generated by
# DBI::DBD::Metadata::write_typeinfo_pm v2.014214.

package DBD::CSV::TypeInfo;

{
    require Exporter;
    require DynaLoader;
    @ISA    = qw(Exporter DynaLoader);
    @EXPORT = qw(type_info_all);
    use DBI qw(:sql_types);

    $type_info_all = [
	{   TYPE_NAME          => 0,
	    DATA_TYPE          => 1,
	    COLUMN_SIZE        => 2,
	    LITERAL_PREFIX     => 3,
	    LITERAL_SUFFIX     => 4,
	    CREATE_PARAMS      => 5,
	    NULLABLE           => 6,
	    CASE_SENSITIVE     => 7,
	    SEARCHABLE         => 8,
	    UNSIGNED_ATTRIBUTE => 9,
	    FIXED_PREC_SCALE   => 10,
	    AUTO_UNIQUE_VALUE  => 11,
	    LOCAL_TYPE_NAME    => 12,
	    MINIMUM_SCALE      => 13,
	    MAXIMUM_SCALE      => 14,
	    SQL_DATA_TYPE      => 15,
	    SQL_DATETIME_SUB   => 16,
	    NUM_PREC_RADIX     => 17,
	    INTERVAL_PRECISION => 18,
	    },
	[   "VARCHAR", SQL_VARCHAR, undef, "'", "'", undef, 0, 1, 1, 0, undef,
	    undef, undef, 1, 999999, undef, undef, undef, undef,
	    ],
	[   "CHAR", SQL_CHAR, undef, "'", "'", undef, 0, 1, 1, 0,
	    undef, undef, undef, 1, 999999, undef, undef, undef, undef,
	    ],
	[   "INTEGER", SQL_INTEGER, undef, "", "", undef, 0, 0, 1, 0, undef,
	    undef, undef, 0, 0, undef, undef, undef, undef,
	    ],
	[   "REAL", SQL_REAL, undef, "",    "",    undef,
	    0,      0,        1,     0,     undef, undef,
	    undef,  0,        0,     undef, undef, undef,
	    undef,
	    ],
	[   "BLOB", SQL_LONGVARBINARY, undef, "'", "'", undef, 0, 1, 1, 0,
	    undef, undef, undef, 1, 999999, undef, undef, undef, undef,
	    ],
	[   "BLOB", SQL_LONGVARBINARY, undef, "'", "'", undef, 0, 1, 1, 0,
	    undef, undef, undef, 1, 999999, undef, undef, undef, undef,
	    ],
	[   "TEXT", SQL_LONGVARCHAR, undef, "'", "'", undef, 0, 1, 1, 0, undef,
	    undef, undef, 1, 999999, undef, undef, undef, undef,
	    ],
	];

    1;
    }
